@@ -1,6 +1,6 @@
 /*-------------------------------------------------------------------------*/
 /* elements.c --- xcircuit routines for creating basic on-screen elements  */
-/* Copyright (c) 1998  Tim Edwards, Johns Hopkins University       	   */
+/* Copyright (c) 1998  Tim Edwards, Johns Hopkins University        	   */
 /*-------------------------------------------------------------------------*/
 
 /*-------------------------------------------------------------------------*/
@@ -62,7 +62,11 @@ double saveratio;
 /*------------------------------------------------------------------------*/
 
 extern void undrawtext(), redrawtext();
-XtEventHandler trackspline(), trackarc(), trackbox();
+extern XtEventHandler trackspline(), trackarc(), trackbox();
+
+#ifdef SCHEMA
+extern labelptr findotherpin();
+#endif
 
 /*----------------------*/
 /* Start spline mode.   */
@@ -82,13 +86,17 @@ void startspline(button, clientdata, calldata)
 /* Start label mode.    */
 /*----------------------*/
 
-void starttext(button, clientdata, calldata)
+void starttext(button, mode, calldata)
   Widget       button;
-  caddr_t      clientdata, calldata;
+  unsigned int mode;
+  caddr_t      calldata;
 {
    if (eventmode == NORMAL_MODE) {
       Wprintf("Click button1 for text.");
-      eventmode = TEXT0_MODE;
+      if (mode == 0)
+         eventmode = TEXT0_MODE;
+      else
+	 eventmode = TEXT1_MODE;	/* Pinlabel */
    }
 }
 
@@ -96,9 +104,9 @@ void starttext(button, clientdata, calldata)
 /* Button handler when creating a label */
 /*--------------------------------------*/
 
-void textbutton(w, clientdata, event)
+void textbutton(w, dopin, event)
   Widget w;
-  caddr_t  clientdata;
+  uchar dopin;
   XButtonEvent *event;
 {
    labelptr *newlabel;
@@ -122,6 +130,14 @@ void textbutton(w, clientdata, event)
    *((*newlabel)->string + 1) = (char)(areastruct.psfont + FONT_START);
    *((*newlabel)->string + 2) = '\0';
 
+#ifdef SCHEMA
+   (*newlabel)->pin = False;
+   if (dopin == True && areastruct.schemon) {
+      (*newlabel)->pin = LOCAL;
+      (*newlabel)->color = SNAPCOLOR;
+   }
+#endif
+
    (*newlabel)->justify = areastruct.justify;
    textpos = 2;
 
@@ -207,11 +223,18 @@ void labeltext(keypressed, keystate)
    char report[2][50];
    XPoint cursorpt;
    short tmplength, tmpheight, cfont, ncfont;
+#ifdef SCHEMA
+   labelptr symlabel;
+#endif
 
    curlabel = TOLABEL(EDITPART);
    for (endpos = curlabel->string; *endpos != '\0'; endpos++);
    curpos = curlabel->string + textpos;
 
+#ifdef SCHEMA
+   symlabel = findotherpin(curlabel);
+#endif
+
    /* find current font */
 
    for (sptr = curpos - 2; sptr >= curlabel->string; sptr--)
@@ -317,14 +340,14 @@ void labeltext(keypressed, keystate)
       else return;
    }
 
-   if (keypressed == XK_BackSpace)
+   else if (keypressed == XK_BackSpace)
       keypressed = XK_Delete;
 
    if (keypressed == XK_Delete && curpos > curlabel->string + 2){
-	 short skip = (*(curpos - 2) == TEXT_ESC) ? 2 : 1;
-	 undrawtext(curlabel);
-	 textpos -= skip;
-	 for (; curpos <= endpos; curpos++) *(curpos-skip) = *curpos;
+      short skip = (*(curpos - 2) == TEXT_ESC) ? 2 : 1;
+      undrawtext(curlabel);
+      textpos -= skip;
+      for (; curpos <= endpos; curpos++) *(curpos-skip) = *curpos;
    }
    else if (keypressed == XK_Return) {
       Boolean hasstuff = False;   /* Check for null string */
@@ -431,6 +454,11 @@ void labeltext(keypressed, keystate)
       *(curlabel->string + slen + 1) = '\0';
    }
 
+#ifdef SCHEMA
+   if (symlabel != NULL) symlabel->string = curlabel->string;
+#endif
+
+
    if ((keypressed > 0 && keypressed < 256) || keypressed == XK_Delete ||
 	keypressed == XK_Right || keypressed == XK_Left || (keystate == 1
 	&& (keypressed == XK_A || keypressed == XK_B)) || keystate == 2) {
@@ -983,7 +1011,6 @@ void trackpoly(w, clientdata, calldata)
 	    if (savedir & LASTX) bpt->x += deltax;
 	    if (savedir & LASTY) bpt->y += deltay;
 	 }
-
 	 if (fpt != NULL) {
 	    if (savedir & NEXTX) fpt->x += deltax;
 	    if (savedir & NEXTY) fpt->y += deltay;
