@@ -1,6 +1,6 @@
 /*-----------------------------------------------------------------------*/
 /* files.c --- file handling routines for xcircuit			 */
-/* Copyright (c) 1998  Tim Edwards, Johns Hopkins University       	 */
+/* Copyright (c) 1998  Tim Edwards, Johns Hopkins University        	 */
 /*-----------------------------------------------------------------------*/
 
 #include <stdio.h>
@@ -59,10 +59,15 @@ extern colorindex *colorlist;
 /* Declarations of functions defined externally to functions.c            */
 /*------------------------------------------------------------------------*/
 
+#ifdef SCHEMA
+extern void checkschem();
+#endif
+
 void savefile(), startloadfile(), importfile(), loadfile(), printobjects();
 void loadlibrary(), savelibrary();
 Boolean objectread();
 float getpsscale();
+
 float version;
 
 extern void calcbbox(), setgridtype();
@@ -127,6 +132,7 @@ void topreset()
    areastruct.wirewidth[rpage] = 2.0;
    areastruct.orient[rpage] = 0;
    areastruct.pmode[rpage] = 0;
+   areastruct.outscale[rpage] = 1.0;
    areastruct.filename[rpage] = (char *)malloc((1 + strlen((*(areastruct.pagelist
 	+ rpage))->name)) * sizeof(char));
    strcpy(areastruct.filename[rpage], (*(areastruct.pagelist + rpage))->name);
@@ -134,21 +140,19 @@ void topreset()
    areastruct.drawingscale[rpage].x = areastruct.drawingscale[rpage].y = 1;
    areastruct.pcorner[rpage].x = -areastruct.width;
    areastruct.pcorner[rpage].y = -areastruct.height;
-   areastruct.outscale[rpage] = 1.0;
 
    /* use the coordstyle of page 0 as the default for new pages */
 
    if (areastruct.coordstyle[0] == CM) {
       areastruct.coordstyle[rpage] = CM;
-      areastruct.pagesize[rpage].x = 595;	 /* A4 */
-      areastruct.pagesize[rpage].y = 842;
+      areastruct.pagesize[rpage].x = 595;
+      areastruct.pagesize[rpage].y = 842;  /* A4 */
    }
    else {
       areastruct.coordstyle[rpage] = FRAC_INCH;
-      areastruct.pagesize[rpage].x = 612;	 /* letter */
+      areastruct.pagesize[rpage].x = 612;	/* letter */
       areastruct.pagesize[rpage].y = 792;
    }
-
 }
 
 void initmem(localdata)
@@ -156,6 +160,15 @@ void initmem(localdata)
 {
    localdata->parts = 0;
    localdata->plist = (genericptr *)malloc(sizeof(genericptr));
+
+#ifdef SCHEMA
+   localdata->schemtype = SCHEMATIC;
+   localdata->symschem = NULL;
+   localdata->netlist = NULL;
+   localdata->pinlist = NULL;
+   localdata->calllist = NULL;
+   localdata->traversed = 0;
+#endif
 }
 
 /*--------------------------------------------------------------*/
@@ -293,7 +306,7 @@ float getpsscale(value, page)
   short page;
 {
    if (areastruct.coordstyle[page] == FRAC_INCH ||
-         areastruct.coordstyle[page] == DEC_INCH)
+ 	 areastruct.coordstyle[page] == DEC_INCH)
       return (value * INCHSCALE);
    else
       return (value * CMSCALE);
@@ -410,7 +423,7 @@ void loadlibrary(mode)
          Wprintf("Error in library.");
          return;
       }
-      sscanf(temp, "%c %.19s", &percentc, keyword);
+      sscanf(temp, "%c %s", &percentc, keyword);
 	    if(percentc == '%' && !strcmp(keyword, "XCircuitLib")) break;
    }
    objectread(ps, objectdata, 0, 0, mode, temp, DEFAULTCOLOR);
@@ -516,7 +529,7 @@ void loadfile(mode)
 	 Wprintf("Error: EOF in or before prolog.");
 	 return;
       }
-      sscanf(temp, "%c%.19s", &percentc, keyword);
+      sscanf(temp, "%c%s", &percentc, keyword);
       for (pdchar = keyword; isspace(*pdchar); pdchar++);
       if (percentc == '%') {
 	 if (!strcmp(pdchar, "XCircuit")) break;
@@ -655,6 +668,13 @@ void loadfile(mode)
          renamepage(areastruct.page);
       }
 
+#ifdef SCHEMA
+      /* If schematics are enabled, if name is the same as a library	*/
+      /* object, then this is its schematic.				*/
+
+      checkschem();
+#endif
+
       /* rotation (landscape mode) is optional; parse accordingly */
 
       sscanf(temp, "%f %s", &tmpfl, teststr);
@@ -736,7 +756,7 @@ Boolean objectread(ps, localdata, offx, offy, mode, retstr, ccolor)
   char		*retstr;
   int		ccolor;
 {
-   char *temp, *buffer, keyword[80]; 
+   char *temp, *buffer, keyword[30]; 
    short tmpfont = 0;
    int bufsize = 256;
    float tmpscale = 0.0;
@@ -769,7 +789,7 @@ Boolean objectread(ps, localdata, offx, offy, mode, retstr, ccolor)
       if (lineptr != buffer) {  /* ignore any blank lines */
          for (keyptr = lineptr - 1; isspace(*keyptr) && keyptr != buffer; keyptr--);
          for (; !isspace(*keyptr) && keyptr != buffer; keyptr--);
-         sscanf(keyptr, "%.79s", keyword);
+         sscanf(keyptr, "%s", keyword);
 
          if (!strcmp(keyword, "showpage")) {
             strncpy(retstr, buffer, 150);
@@ -1075,12 +1095,22 @@ Boolean objectread(ps, localdata, offx, offy, mode, retstr, ccolor)
 	    if (i == fontcount) i = 0;	/* Why bother with anything fancy? */
          }
 
-         else if (!strcmp(keyword, "label")) {
+         else if (!strcmp(keyword, "label")
+#ifdef SCHEMA
+                || !strcmp(keyword, "pinlabel")
+                || !strcmp(keyword, "pinglobal")
+
+#endif
+					        ){
 	    labelptr *newlabel;
 	    short segs, i, j;
 	    uchar tmpstring[150], *tmpptr = tmpstring;
 	    char *sptr;
 
+#ifdef SCHEMA
+	    /* ignore pinlabels if we are not in schematic capture mode */
+	    if ((!strcmp(keyword, "label")) || areastruct.schemon == True) {
+#endif
 	    NEW_LABEL(newlabel, localdata);
 
 	    /* scan backwards to get the number of substrings */
@@ -1099,6 +1129,15 @@ Boolean objectread(ps, localdata, offx, offy, mode, retstr, ccolor)
 	    (*newlabel)->rotation++;
 	    (*newlabel)->scale = tmpscale;
 	    (*newlabel)->color = curcolor;
+#ifdef SCHEMA
+	    (*newlabel)->pin = False;
+	    if (areastruct.schemon) {
+	       if (!strcmp(keyword, "pinlabel"))
+		  (*newlabel)->pin = LOCAL;
+	       else if (!strcmp(keyword, "pinglobal"))
+		  (*newlabel)->pin = GLOBAL;
+	    }
+#endif
 
 	    for (; *lineptr == ' '; lineptr--);
 
@@ -1149,14 +1188,14 @@ Boolean objectread(ps, localdata, offx, offy, mode, retstr, ccolor)
 		   *tmpptr++ = TEXT_ESC;
 		   *tmpptr++ = NORMALSCRIPT;
 	       }
-	       else if (!strcmp(lineptr + 1, "qS")) {
-		   *tmpptr++ = TEXT_ESC;
-		   *tmpptr++ = QTRSPACE;
-	       }
 	       else if (!strcmp(lineptr + 1, "hS")) {
 		   *tmpptr++ = TEXT_ESC;
 		   *tmpptr++ = HALFSPACE;
 	       }
+	       else if (!strcmp(lineptr + 1, "qS")) {
+		   *tmpptr++ = TEXT_ESC;
+		   *tmpptr++ = QTRSPACE;
+	       }
 	       else if (*(lineptr + 1) == '(')   /* "bs" for backspace assumed */
 		  for (sptr = lineptr + 2; *sptr != ')'; sptr++) {
 		     *tmpptr++ = TEXT_ESC;
@@ -1232,7 +1271,32 @@ Boolean objectread(ps, localdata, offx, offy, mode, retstr, ccolor)
                sizeof(uchar));
 	    strcpy((*newlabel)->string, tmpstring);
 
+#ifdef SCHEMA
+	    /* link string to the one that corresponds to this one */
+	    /* in the symbol, if it exists.			   */
+
+	    if (areastruct.schemon && localdata->symschem != NULL &&
+		  (*newlabel)->pin) {
+	       genericptr *pgen;
+	       for (pgen = localdata->symschem->plist; pgen < localdata->
+			symschem->plist + localdata->symschem->parts; pgen++) {
+		  if ((*pgen)->type == LABEL)
+		     if (!strcmp(TOLABEL(pgen)->string, (*newlabel)->string)) {
+			free((*newlabel)->string);
+			(*newlabel)->string = TOLABEL(pgen)->string;
+		 	break;
+		     }
+	       }
+	        if (pgen == localdata->symschem->plist + localdata->symschem->parts)
+		   Wprintf("Error:  Unattached pin");
+	    }
+#endif
+
 	    localdata->parts++;
+#ifdef SCHEMA
+	    /* endif: ignore pinlabels if we are not in schematic capture mode */
+	    }
+#endif
          }
 
          /* read bounding box */
@@ -1248,6 +1312,12 @@ Boolean objectread(ps, localdata, offx, offy, mode, retstr, ccolor)
 	    sscanf(++lineptr, "%hd %hd %hd %hd", &localdata->lowerleft.x, 
 	       &localdata->lowerleft.y, &localdata->width, 
 	       &localdata->height);
+#ifdef SCHEMA
+	    localdata->lleft2.x = localdata->lowerleft.x;
+	    localdata->lleft2.y = localdata->lowerleft.y;
+	    localdata->width2 = localdata->width;
+	    localdata->height2 = localdata->height;
+#endif
          }
 
 	 /* read "hidden" attribute */
@@ -1273,7 +1343,7 @@ Boolean objectread(ps, localdata, offx, offy, mode, retstr, ccolor)
                *retstr = '\0';
 	       return True;
 	    }
-	    sscanf(lineptr, "%.79s", keyword);
+	    sscanf(lineptr, "%s", keyword);
 
 	    curlib = (objectptr *) realloc(curlib, (*libobjects + 1)
 		      * sizeof(objectptr));
@@ -1298,6 +1368,10 @@ Boolean objectread(ps, localdata, offx, offy, mode, retstr, ccolor)
 	       }
 	    }
 
+#ifdef SCHEMA
+	    (*newobject)->schemtype = SYMBOL;
+	    (*newobject)->symschem = NULL;
+#endif
             (*libobjects)++;
 	    sprintf((*newobject)->name, "%s", keyword);
 	    if (objectread(ps, *newobject, 0, 0, mode, retstr, curcolor) == True) {
@@ -1334,6 +1408,10 @@ Boolean objectread(ps, localdata, offx, offy, mode, retstr, ccolor)
 	    sscanf (lineptr + 1, "%s", _STR);
 	    loadlibrary(mode);
 	 }
+#ifdef SCHEMA
+	 else if (!strcmp(keyword, "fundamental"))
+	    localdata->schemtype = FUNDAMENTAL;
+#endif
          else if (!strcmp(keyword, "EndLib")) break;
 	 else if (!strcmp(keyword, "restore"));    /* handled at top */
 	 else if (!strcmp(keyword, "grestore"));   /* ignore */
@@ -1419,7 +1497,7 @@ void setfile(button, fnamewidget, clientdata)
 {
    /* see if name has been changed in the buffer */
 
-   strcpy(_STR2, (char *)XwTextCopyBuffer(fnamewidget));
+   sprintf(_STR2, "%.249s", (char *)XwTextCopyBuffer(fnamewidget));
    if (strcmp(areastruct.filename[areastruct.page], _STR2)) {
       Wprintf("Changing name of edit file.");
       areastruct.filename[areastruct.page] = (char *) realloc (
@@ -1533,7 +1611,7 @@ void savefile(localdata)
 
    /* calculate the bounding box of the drawing */
 
-   calcbbox(areastruct.topobject);
+   calcbbox(objectdata);
 
    /* find margins */
 
@@ -1553,6 +1631,17 @@ void savefile(localdata)
       }
    }
 
+#ifdef SCHEMA
+   /* find all the schematics which are subcircuits of the top-level-page */
+   /* circuit and give them the same filename, so that they will be saved */
+   /* together as a set.						  */
+
+   if (areastruct.schemon) {
+      Wprintf("Gathering all subcircuits. . .");
+      findsubschems(areastruct.page, objectdata);
+   }
+#endif
+
    /* check for multiple-page output: get the number of pages */
 
    multipage = 0;
@@ -1775,13 +1864,23 @@ void printobjects(ps, localdata, psnorm, xmargin, ymargin, page, mpage,
          fprintf (ps, "%%%%Page: %s %d\n", localdata->name, page);
      
       fprintf (ps, "/pgsave save def bop\n");
+
+#ifdef SCHEMA
+#define lowerleft lleft2
+#define height height2
+#define _width width2
+#else
+#define _width width
+#endif
+
       fprintf(ps, "%% %hd %hd offsets\n", (int)(xmargin / psscale) - localdata->
 	    lowerleft.x, (int)(ymargin / psscale) - localdata->lowerleft.y);
-
       if (areastruct.orient[mpage] == 90)
          fprintf(ps, "90 rotate %d %d translate\n", (int)(ymargin - xmargin),
 	     -((int)((float)localdata->height * psscale) + 
 	     (int)(xmargin + ymargin)));
+
+
       fprintf(ps, "%5.4f ", psnorm);
       switch(areastruct.coordstyle[mpage]) {
 	 case FRAC_INCH: case DEC_INCH:
@@ -1815,11 +1914,22 @@ void printobjects(ps, localdata, psnorm, xmargin, ymargin, page, mpage,
       if (psscale == 0) {
 	 fprintf(ps, "/%s {\n", localdata->name);
 	 fprintf(ps, "%% %hd %hd %hd %hd bbox\n", localdata->lowerleft.x,
-	    localdata->lowerleft.y, localdata->width, localdata->height);
+	    localdata->lowerleft.y, localdata->_width, localdata->height);
 	 if (localdata->hidden == True) fprintf(ps, "%% hidden\n");
 	 fprintf(ps, "begingate\n");
       }
 
+#ifdef SCHEMA
+      /* specify whether this is a fundamental circuit schematic type */
+      if (areastruct.schemon && localdata->schemtype == FUNDAMENTAL) {
+	 fprintf(ps, "%% fundamental\n");
+      }
+
+#undef lowerleft
+#undef height
+#endif
+#undef _width
+
       /* write all the elements in order */
 
       for (savegen = localdata->plist; savegen < localdata->plist +
@@ -2074,14 +2184,14 @@ void printobjects(ps, localdata, psnorm, xmargin, ymargin, page, mpage,
 		        sprintf(_STR, "{ol} ");
 		        dostcount (ps, &stcount, 5);
 	             }
-		     else if (control == HALFSPACE) {
-			sprintf(_STR, "{hS} ");
-			dostcount (ps, &stcount, 5);
-		     }
-		     else if (control == QTRSPACE) {
-			sprintf(_STR, "{qS} ");
-			dostcount (ps, &stcount, 5);
-		     }
+	             else if (control == HALFSPACE) {
+		        sprintf(_STR, "{hS} ");
+		        dostcount (ps, &stcount, 5);
+	             }
+	             else if (control == QTRSPACE) {
+		        sprintf(_STR, "{qS} ");
+		        dostcount (ps, &stcount, 5);
+	             }
 	             else if (control == BACKSPACE) {
 		        short bsctr = 1;
 		
@@ -2102,7 +2212,7 @@ void printobjects(ps, localdata, psnorm, xmargin, ymargin, page, mpage,
 	             else if (control >= FONT_START) {
 		        char *chtmp, *lastfont = fonts[control - FONT_START];
 		        for (chtmp = chrptr - 2; chtmp > (char *)tmpstring; chtmp--)
-		           if ((short)(*chtmp - 1) == TEXT_ESC && (short)(*chtmp)
+		           if ((short)(*chtmp - 1) == TEXT_ESC && (short)*chtmp
 			      >= FONT_START) break;
 		        if (chtmp <= (char *)tmpstring + 1) {
 		           sprintf(_STR, "{/%s %5.3f cf} ", lastfont, 
@@ -2121,9 +2231,23 @@ void printobjects(ps, localdata, psnorm, xmargin, ymargin, page, mpage,
 	       rotation = 360 - (TOLABEL(savegen)->rotation - 1) * ROT_UNIT;
 	       if (rotation == 360) rotation = 0; 
 	       if (segs > 0) {
-                  sprintf(_STR, "%hd %hd %hd %hd %hd label\n", segs,
+                  sprintf(_STR, "%hd %hd %hd %hd %hd ", segs,
 		      TOLABEL(savegen)->justify, rotation, xmat(TOLABEL(savegen)
 		      ->position.x), ymat(TOLABEL(savegen)->position.y));
+
+#ifdef SCHEMA
+		  if (areastruct.schemon) {
+		     if (TOLABEL(savegen)->pin == LOCAL)
+			strcat(_STR, "pinlabel\n");
+	 	     else if (TOLABEL(savegen)->pin == GLOBAL)
+			strcat(_STR, "pinglobal\n");
+		     else strcat(_STR, "label\n");
+		  }
+		  else strcat(_STR, "label\n");
+#else
+		  strcat(_STR, "label\n");
+#endif
+
 	          stcount += strlen(_STR);
 	          if (stcount > OUTPUTWIDTH) fprintf(ps, "\n");
 	          fputs(_STR, ps);
