@@ -1,10 +1,11 @@
 /*-----------------------------------------------------------------------*/
 /* formats.c --- input file support for xcircuit			 */
-/* Copyright (c) 1998  Tim Edwards, Johns Hopkins University       	 */
+/* Copyright (c) 1998  Tim Edwards, Johns Hopkins University        	 */
 /*-----------------------------------------------------------------------*/
 
 #include <stdio.h>
-#include <strings.h>
+#include <string.h>
+#include <malloc.h>
 #include <X11/Intrinsic.h>
 #include <X11/StringDefs.h>
 
@@ -38,15 +39,9 @@ extern short pushes;
 /* Declarations of functions defined externally to functions.c            */
 /*------------------------------------------------------------------------*/
 
-extern void calcbbox(), zoomview(), reset(), topreset(), initmem();
+extern void zoomview(), reset(), topreset(), initmem();
 extern Boolean nextfilename();
 
-/*-------------------------*/
-#ifdef LGF
-void loadlgf();
-void startloadlgf();
-#endif
-
 /*-------------------------*/
 #ifdef LGF
 
@@ -90,16 +85,27 @@ void loadlgf(nullval)
       }
    }
 
-   /* for PostScript file, remove ".lgf" or ".lfo" and replace with ".ps" */
-
-   if ((pdchar = strstr(_STR, ".l")) != NULL) *pdchar = '\0';
-   sprintf(objectdata->name, "%s.ps", _STR);
    sprintf(_STR, "Loaded file: %s", inname);
    Wprintf(_STR);
+
+   /* for PostScript file, remove ".lgf" or ".lfo" and replace with ".ps" */
+
+   if ((pdchar = strstr(inname, ".l")) != NULL) *pdchar = '\0';
+
+   areastruct.filename[areastruct.page] = (char *) realloc (
+      areastruct.filename[areastruct.page], (strlen(inname) + 1) * sizeof(char));
+   strcpy(areastruct.filename[areastruct.page], inname);
+
+   /* If the filename has a path component, use only the root */
+
+   if ((pdchar = strrchr(inname, '/')) != NULL)
+      sprintf(objectdata->name, "%s", pdchar + 1);
+   else
+      sprintf(objectdata->name, "%s", inname);
+
    renamepage(areastruct.page);
    printname(objectdata);
 
-
    /* Make sure that LGF object library has been loaded by loading it now. */
 
    if (!lgfisloaded) {
@@ -444,6 +450,9 @@ void loadlgf(nullval)
 
 			   NEW_LABEL(newlabel, objectdata);
 			   objectdata->parts++;
+			   /* reconnect newinst if displaced by realloc() */
+			   newinst = (objinstptr *)(objectdata->plist
+				+ objectdata->parts - 2);
 
 			   (*newlabel)->position.x = (*newinst)->position.x;
 			   if (!strcmp(tmpstring, "TO"))
@@ -549,12 +558,6 @@ void loadlgf(nullval)
 #endif
 /* (LGF) */
 
-/*-------------------------*/
-#ifdef MATLAB_4X
-void loadmat4();
-void startloadmat4();
-#endif
-
 /*-------------------------*/
 #ifdef MATLAB_4X
 
