@@ -1,6 +1,6 @@
 /*-------------------------------------------------------------------------*/
 /* functions.c 								   */
-/* Copyright (c) 1998  Tim Edwards, Johns Hopkins University       	   */
+/* Copyright (c) 1998  Tim Edwards, Johns Hopkins University        	   */
 /*-------------------------------------------------------------------------*/
 
 /*-------------------------------------------------------------------------*/
@@ -766,10 +766,12 @@ calcextents(bboxgen, llx, lly, urx, ury)
    }
 }
 
+/*-------------------------------------------------------------------------*/
+/* Bounding box calculation for an object				   */
 /*-------------------------------------------------------------------------*/
 
 void calcbbox(localdata)
-  objinstptr	localdata;
+  objectptr	localdata;
 {
    /* set starting bounds as maximum bounds of screen */
 
@@ -779,22 +781,29 @@ void calcbbox(localdata)
 
    short llx, lly, urx, ury;
 
+#ifdef SCHEMA
+   short pllx, plly, purx, pury;  /* pin label extents */
+
+   pllx = plly = 32767;
+   purx = pury = -32768;
+#endif
+
    llx = lly = 32767;
    urx = ury = -32768;
 
    /* now, for each screen element, compute the extents and revise bounding
       box points, if necessary. */
 
-   for (bboxgen = localdata->thisobject->plist; bboxgen < localdata->
-	thisobject->plist + localdata->thisobject->parts; bboxgen++) {
+   for (bboxgen = localdata->plist; bboxgen < localdata->plist +
+		localdata->parts; bboxgen++) {
       switch((*bboxgen)->type) {
 
 	 case(OBJECT):
             points[0].x = points[1].x = TOOBJINST(bboxgen)->thisobject->lowerleft.x;
-            points[1].y = points[2].y = TOOBJINST(bboxgen)->thisobject->lowerleft.y + 
-	       TOOBJINST(bboxgen)->thisobject->height;
-            points[2].x = points[3].x = TOOBJINST(bboxgen)->thisobject->lowerleft.x + 
-	       TOOBJINST(bboxgen)->thisobject->width;
+            points[1].y = points[2].y = TOOBJINST(bboxgen)->thisobject->lowerleft.y
+		+ TOOBJINST(bboxgen)->thisobject->height;
+            points[2].x = points[3].x = TOOBJINST(bboxgen)->thisobject->lowerleft.x
+		+ TOOBJINST(bboxgen)->thisobject->width;
             points[0].y = points[3].y = TOOBJINST(bboxgen)->thisobject->lowerleft.y;
             UTransformPoints(points, npoints, 4, TOOBJINST(bboxgen)->position,
 		 TOOBJINST(bboxgen)->scale, TOOBJINST(bboxgen)->rotation); 
@@ -815,10 +824,24 @@ void calcbbox(localdata)
 
             UTransformPoints(points, npoints, 4, TOLABEL(bboxgen)->position,
 		TOLABEL(bboxgen)->scale, TOLABEL(bboxgen)->rotation); 
+#ifdef SCHEMA
+	    /* separate bounding box for pinlabels */
+
+	    if (TOLABEL(bboxgen)->pin == True) {
+               for (j = 0; j < 4; j++) {
+                  bboxcalc(npoints[j].x, &pllx, &purx);
+                  bboxcalc(npoints[j].y, &plly, &pury);
+               }
+	    }
+	    else {
+#endif
             for (j = 0; j < 4; j++) {
                bboxcalc(npoints[j].x, &llx, &urx);
                bboxcalc(npoints[j].y, &lly, &ury);
             }
+#ifdef SCHEMA
+	    }
+#endif
             } break;
 
 	 case(PATH): {
@@ -835,29 +858,45 @@ void calcbbox(localdata)
    
    /* default action if there are no elements */
 
-   if (localdata->thisobject->parts == 0) {
+   if (localdata->parts == 0) {
       llx = areastruct.lowerleft->x;
       lly = areastruct.lowerleft->y;
       urx = llx + areastruct.width / (*areastruct.vscale);
       ury = lly + areastruct.height / (*areastruct.vscale);
    }
 
+#ifdef SCHEMA
+   /* bounding box without pin info. . */
+
+   localdata->lleft2.x = llx;
+   localdata->lleft2.y = lly;
+   localdata->width2 = urx - llx;
+   localdata->height2 = ury - lly;
+
+   /* . . . and with . . . */
+
+   if (pllx < llx) llx = pllx;
+   if (plly < lly) lly = plly;
+   if (purx > urx) urx = purx;
+   if (pury > ury) ury = pury;
+#endif
+
    /* set the new bounding box */
 
-   localdata->thisobject->lowerleft.x = llx;
-   localdata->thisobject->lowerleft.y = lly;
-   localdata->thisobject->width = urx - llx;
-   localdata->thisobject->height = ury - lly;
+   localdata->lowerleft.x = llx;
+   localdata->lowerleft.y = lly;
+   localdata->width = urx - llx;
+   localdata->height = ury - lly;
 
    /* avoid width or height of zero---use of TEXTHEIGHT is arbitrary. . . */
 
-   if (localdata->thisobject->width == 0) {
-      localdata->thisobject->width = TEXTHEIGHT;
-      localdata->thisobject->lowerleft.x -= HALFHEIGHT;
+   if (localdata->width == 0) {
+      localdata->width = TEXTHEIGHT;
+      localdata->lowerleft.x -= HALFHEIGHT;
    }
-   if (localdata->thisobject->height == 0) {
-      localdata->thisobject->height = TEXTHEIGHT;
-      localdata->thisobject->lowerleft.y -= HALFHEIGHT;
+   if (localdata->height == 0) {
+      localdata->height = TEXTHEIGHT;
+      localdata->lowerleft.y -= HALFHEIGHT;
    }
 }
 
@@ -870,7 +909,7 @@ void zoomview(w, mode, nulldata)
   unsigned short mode;
   caddr_t	 nulldata;
 {
-   calcbbox(areastruct.topobject);
+   calcbbox(objectdata);
 
    if (eventmode == NORMAL_MODE || eventmode == POPUP_MODE ||
 	 eventmode == COPY2_MODE || eventmode == PUSH_MODE ||
@@ -931,6 +970,21 @@ void UDrawLine(localdata, pt1, pt2)
    UDrawSimpleLine(localdata, pt1, pt2); 
 } 
 
+/*-------------------------------------------------------------------------*/
+/* Add "X" at string origin						   */
+/*-------------------------------------------------------------------------*/
+
+void UDrawX(curlabel)
+  labelptr curlabel;
+{
+   XPoint upt;
+
+   user_to_window(curlabel->position, &upt);
+   XSetLineAttributes(dpy, areastruct.gc, 0, LineSolid, CapButt, JoinMiter);
+   XDrawLine(dpy, win, areastruct.gc, upt.x - 3, upt.y - 3, upt.x + 3, upt.y + 3);
+   XDrawLine(dpy, win, areastruct.gc, upt.x + 3, upt.y - 3, upt.x - 3, upt.y + 3);
+}
+
 /*-------------------------------------------------------------------------*/
 /* Draw line for editing text (position of cursor in string is given by    */
 /*   areastruct.origin.(x,y)						   */
@@ -963,13 +1017,7 @@ void UDrawTLine(curlabel)
    UDrawLine(areastruct.topobject, &points[0], &points[1]);
    UPopCTM();
 
-   /* Add "X" at string origin */
-
-   user_to_window(curlabel->position, &upt);
-   XSetLineAttributes(dpy, areastruct.gc, 0, LineSolid, CapButt, JoinMiter);
-   XDrawLine(dpy, win, areastruct.gc, upt.x - 3, upt.y - 3, upt.x + 3, upt.y + 3);
-   XDrawLine(dpy, win, areastruct.gc, upt.x + 3, upt.y - 3, upt.x - 3, upt.y + 3);
-
+   UDrawX(curlabel);
 }
 
 /*-------------------------------------------------------------------------*/
@@ -1272,6 +1320,10 @@ void UDrawObject(localdata, theobject, level, passcolor)
 	     break;
    
   	  case(LABEL): 
+#ifdef SCHEMA
+	     if (areastruct.schemon == False || level == 0 ||
+		    TOLABEL(areagen)->pin == False)
+#endif
              UDrawString(localdata, TOLABEL(areagen));
 	     break;
        }
