@@ -1,6 +1,6 @@
 /*-------------------------------------------------------------------------*/
 /* menudefs.h								   */
-/* Copyright (c) 1998  Tim Edwards, Johns Hopkins University       	   */
+/* Copyright (c) 1998  Tim Edwards, Johns Hopkins University        	   */
 /*-------------------------------------------------------------------------*/
 
 /*-------------------------------------------------------------------------*/
@@ -39,6 +39,9 @@ extern XtCallbackProc getwirewidth(), getwwidth(), getlib(), savelib(), getuserl
 extern XtCallbackProc startdesel(), starthelp(), getdscale(), startspline();
 extern XtCallbackProc getgridtype(), zoombox(), startedit(), startman(), boxedit(); 
 extern XtCallbackProc inversecolor(), addnewcolor(), setcolor();
+#ifdef SCHEMA
+extern XtCallbackProc swapschem(), callgennet(), makefund(), dopinglobal();
+#endif
 
 /*-------------------------------------------------------------------------*/
 /* Hierarchical menus must be listed in bottom-up order			   */
@@ -48,7 +51,7 @@ extern XtCallbackProc inversecolor(), addnewcolor(), setcolor();
 /*		    passed to getfill().				   */
 /*-------------------------------------------------------------------------*/
 
-#define submenu(a) 	a, (sizeof (a) / sizeof(menustruct)), \
+#define submenu(a)	a, (sizeof (a) / sizeof(menustruct)), \
 			(XtCallbackProc)DoNothing, NULL
 #define action(b,c)	NULL, 0, (XtCallbackProc)b, c
 #define setaction(b,c)	NULL, 1, (XtCallbackProc)b, c
@@ -166,10 +169,10 @@ menustruct TextMenu[] = {
 	{"Text Size", action(gettsize, &areastruct.textscale)},
 	{"Font", submenu(Fonts)},
 	{"Style", submenu(FontStyles)},
-	{"Insert Space", submenu(TextSpacing)},
+	{"Insert Space", submenu(TextSpacing)},             
 	{"Justification (keypad)", submenu(Justifs)},
 	{" ", noaction},
-	{"Make Label (t)", action(starttext, NULL)},
+	{"Make Label (t)", action(starttext, Number(0))},
 };
 
 menustruct PolyMenu[] = {
@@ -194,7 +197,7 @@ menustruct RotateMenu[] = {
 	{" ", noaction},
 	{"Rotate CCW 90", action(startrotate, Number(-18))},
 	{"Rotate CCW 45", action(startrotate, Number(-9))},
-	{"Rotate CCW 30", action(startrotate, Number(-6))},
+	{"Rotate CCW 30", action(startrotate, Number(-6))}, 
 	{"Rotate CCW 15 (R)", action(startrotate, Number(-3))},
 	{"Rotate CCW 5 (O)", action(startrotate, Number(-1))},
 };
@@ -211,6 +214,18 @@ menustruct SnapMenu[] = {
         {"Snap spacing", action(getint, &areastruct.snapspace)},
 };
 
+#ifdef SCHEMA
+menustruct SchemaMenu[] = {
+	{"Make Pin (T)", action(starttext, Number(1))},
+	{"Make Pin Global", action(dopinglobal, NULL)},
+	{"Make Object Fundamental", action(makefund, NULL)},
+	{"Switch Symbol/Schematic", action(swapschem, NULL)},
+	{" ", noaction},
+	{"Write spice", action(callgennet, Number(0))},
+	{"Write sim", action(callgennet, Number(1))},
+};
+#endif
+
 menustruct WindowMenu[] = {
 	{"Zoom In (Z)", action(zoomin, Number(0))},
 	{"Zoom Out (z)", action(zoomout, NULL)},
@@ -276,6 +291,9 @@ menustruct TopButtons[] = {
 	{"Text", submenu(TextMenu)},
 	{"Options", submenu(OptionMenu)},
 	{"Window", submenu(WindowMenu)},
+#ifdef SCHEMA
+	{"Schema", submenu(SchemaMenu)},
+#endif
 };
 
 #undef submenu
