@@ -1,6 +1,6 @@
 /*------------------------------------------------------------------------------*/
 /*   menudep.c									*/
-/* Copyright (c) 1998  Tim Edwards, Johns Hopkins University       		*/
+/* Copyright (c) 1998  Tim Edwards, Johns Hopkins University        		*/
 /*------------------------------------------------------------------------------*/
 /* Parse file menudefs.h and create file of definitions for use by		*/
 /* "XtNameToWidget" to find the menu.						*/
@@ -26,8 +26,6 @@
 /*------------------------------------------------------------------------------*/
 
 #include <stdio.h>
-#include <stdlib.h>
-#include <ctype.h>
 #include <string.h>
 
 #define MAXMENUS 200
