@@ -1,6 +1,6 @@
 /*-------------------------------------------------------------------------*/
 /* rcfile.c --- Load initialization variables from the .xcircuitrc file	   */
-/* Copyright (c) 1998  Tim Edwards, Johns Hopkins University       	   */
+/* Copyright (c) 1998  Tim Edwards, Johns Hopkins University        	   */
 /*-------------------------------------------------------------------------*/
    
 /*-------------------------------------------------------------------------*/
@@ -10,6 +10,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <malloc.h>
 
 #include <X11/Intrinsic.h>
 #include <X11/StringDefs.h>
@@ -133,8 +134,6 @@ loadrcfile()
          else if (!strcmp(_STR, "colorscheme")) {
 	    sscanf(temp, "%*s %*s %s", value);
 	    if (!strcmp(value, "inverse") || !strcmp(value, "2")) {
-	       /* make sure inverse-color flag is False so that inversecolor() */
-	       /* will toggle it to true.				       */
 	       areastruct.invert = False;
 	       inversecolor(NULL, &areastruct.invert, NULL);
 	    }
@@ -143,11 +142,11 @@ loadrcfile()
 	    sscanf(temp, "%*s %*s %s", value);
 	    if (!strcmp(value, "cm") || !strcmp(value, "centimeters")) {
 	       areastruct.coordstyle[0] = CM;
-	       areastruct.pagesize[0].x = 595;	 /* A4 size */
+	       areastruct.pagesize[0].x = 595;  /* A4 size */
 	       areastruct.pagesize[0].y = 842;
 	       togglegrid((unsigned short)areastruct.coordstyle[0]);
-	    }
-	 }
+            }
+         }
       }
    }
 
@@ -169,13 +168,12 @@ end:
       addnewcolorentry(xc_alloccolor("Gray90"));
       addnewcolorentry(xc_alloccolor("Red"));
       addnewcolorentry(xc_alloccolor("Blue"));
-      addnewcolorentry(xc_alloccolor("Green3"));
+      addnewcolorentry(xc_alloccolor("Green"));
       addnewcolorentry(xc_alloccolor("Yellow"));
       addnewcolorentry(xc_alloccolor("Purple"));
       addnewcolorentry(xc_alloccolor("Cyan"));
       addnewcolorentry(xc_alloccolor("Magenta"));
       addnewcolorentry(xc_alloccolor("Orange"));
-      addnewcolorentry(xc_alloccolor("Beige"));
       addnewcolorentry(xc_alloccolor("Tan"));
       addnewcolorentry(xc_alloccolor("Brown"));
    }
