@@ -9,6 +9,7 @@
 
 #include <stdio.h>
 #include <string.h>
+#include <malloc.h>
 #include <math.h>
 
 #include <X11/Intrinsic.h>
@@ -180,8 +181,21 @@ Boolean pathselect(curgen, class)
   genericptr *curgen;
   short class;
 {
-   int		sqrwirelim = (4.5 / (*areastruct.vscale)) + 18;
-   int		wirelim = 1.3 + (sqrwirelim + 2) * (sqrwirelim + 2);
+   /*----------------------------------------------------------------------*/
+   /* wirelim is the distance, in user-space units, at which an element is */
+   /* considered for selection.						   */
+   /*									   */
+   /* wirelim = A + B / (scale + C)					   */
+   /*									   */
+   /* where A = minimum possible distance (expands range at close scale)   */
+   /*       C = minimum possible scale    (contract range at far scale)	   */
+   /*	    B   makes wirelim approx. 25 at default scale of 0.5, which	   */
+   /*		is an empirical result.					   */
+   /*----------------------------------------------------------------------*/
+
+   float	wirelim = 2 + 11.5 / (*areastruct.vscale + 0.05);
+   long		sqrwirelim = (int)(wirelim * wirelim);
+
    long		newdist;
    Boolean	selected = False;
 
@@ -201,12 +215,12 @@ Boolean pathselect(curgen, class)
 	 nearpt[0].x = (short)(currentpt->x);
 	 nearpt[0].y = (short)(currentpt->y);
 	 newdist = finddist(&nearpt[0], &nearpt[1], &areastruct.save);
-         if (newdist <= wirelim) break;
+         if (newdist <= sqrwirelim) break;
       }
-      if ((!(TOARC(curgen)->style & UNCLOSED)) && (newdist > wirelim))
+      if ((!(TOARC(curgen)->style & UNCLOSED)) && (newdist > sqrwirelim))
 	 newdist = finddist(&nearpt[0], &nearpt[2], &areastruct.save);
 
-      if (newdist <= wirelim) selected = True;
+      if (newdist <= sqrwirelim) selected = True;
    }
 
    else if ((*curgen)->type == (class & SPLINE)) {
@@ -220,7 +234,7 @@ Boolean pathselect(curgen, class)
       nearpt[0].y = (short)(TOSPLINE(curgen)->points[0].y);
       newdist = finddist(&(TOSPLINE(curgen)->ctrl[0]), &(nearpt[0]),
 		   &areastruct.save);
-      if (newdist > wirelim) {
+      if (newdist > sqrwirelim) {
          for (currentpt = TOSPLINE(curgen)->points; currentpt <
 		  TOSPLINE(curgen)->points + INTSEGS; currentpt++) {
 	    nearpt[1].x = nearpt[0].x;
@@ -228,18 +242,18 @@ Boolean pathselect(curgen, class)
 	    nearpt[0].x = (short)(currentpt->x);
 	    nearpt[0].y = (short)(currentpt->y);
 	    newdist = finddist(&nearpt[0], &nearpt[1], &areastruct.save);
-            if (newdist <= wirelim) break;
+            if (newdist <= sqrwirelim) break;
 	 }
-	 if (newdist > wirelim) {
+	 if (newdist > sqrwirelim) {
 	    newdist = finddist(&nearpt[0], &(TOSPLINE(curgen)->ctrl[3]),
 			&areastruct.save);
-            if ((!(TOSPLINE(curgen)->style & UNCLOSED)) && (newdist > wirelim))
+            if ((!(TOSPLINE(curgen)->style & UNCLOSED)) && (newdist > sqrwirelim))
 	       newdist = finddist(&(TOSPLINE(curgen)->ctrl[0]),
 		     &(TOSPLINE(curgen)->ctrl[3]), &areastruct.save);
 	 }
       }
 
-      if (newdist <= wirelim) selected = True;
+      if (newdist <= sqrwirelim) selected = True;
    }
 
    else if ((*curgen)->type == (class & POLYGON)) {
@@ -251,13 +265,13 @@ Boolean pathselect(curgen, class)
       for (currentpt = TOPOLY(curgen)->points; currentpt < TOPOLY(curgen)
 		->points + TOPOLY(curgen)->number - 1; currentpt++) {
 	 newdist = finddist(currentpt, currentpt + 1, &areastruct.save);
-	 if (newdist <= wirelim) break;
+	 if (newdist <= sqrwirelim) break;
       }
-      if ((!(TOPOLY(curgen)->style & UNCLOSED)) && (newdist > wirelim))
+      if ((!(TOPOLY(curgen)->style & UNCLOSED)) && (newdist > sqrwirelim))
 	 newdist = finddist(currentpt, TOPOLY(curgen)->points,
 		&areastruct.save);
 
-      if (newdist <= wirelim) selected = True;
+      if (newdist <= sqrwirelim) selected = True;
    }
    return selected;
 }
@@ -281,7 +295,6 @@ short *objectselect(class)
    long		newdist;
    Arg		wargs[2];
 
-   XUngrabButton(dpy, Button1, AnyModifier, win);
    XDefineCursor(dpy, win, QUESTION);
 
    /* Definition for unselecting an element */
@@ -307,6 +320,12 @@ short *objectselect(class)
 
       if ((*curgen)->type == (class & OBJECT)) {
 
+#ifdef SCHEMA
+#define lowerleft lleft2
+#define width width2
+#define height height2
+#endif
+
          bboxpts[0].x = bboxpts[1].x = TOOBJINST(curgen)->thisobject->lowerleft.x;
          bboxpts[2].x = bboxpts[3].x = TOOBJINST(curgen)->thisobject->lowerleft.x + 
 	    TOOBJINST(curgen)->thisobject->width;
@@ -316,6 +335,11 @@ short *objectselect(class)
          UTransformPoints(bboxpts, newboxpts, 4, TOOBJINST(curgen)->position,
 		TOOBJINST(curgen)->scale, TOOBJINST(curgen)->rotation);
 
+#ifdef SCHEMA
+#undef lowerleft
+#undef width
+#undef height
+#endif
          /* Look for an intersect of the boundingbox and pointer position. */
          /* This is a simple matter of rotating the pointer position with  */
          /*  respect to the origin of the bounding box segment, as if the  */
@@ -474,10 +498,6 @@ short *objectselect(class)
       }  break;
    }
 
-   XGrabButton(dpy, Button1, AnyModifier, win, True, ButtonPressMask |
-	ButtonMotionMask | ButtonReleaseMask, GrabModeAsync, GrabModeAsync,
-	win, ARROW);
-
    if (found > 1) {
       short *nselect;
 
@@ -486,6 +506,7 @@ short *objectselect(class)
 	 Wprintf("Click and hold button 1 to continue move.");
       else
 	 Wprintf("Selection complete");
+
       for(;;) {
 	 XNextEvent(dpy, &event); 
 	 if (event.type != ButtonPress && event.type != ButtonRelease &&
