@@ -1,6 +1,6 @@
 /*-----------------------------------------------------------------------*/
 /* text.c --- text processing routines for xcircuit		 	 */
-/* Copyright (c) 1998  Tim Edwards, Johns Hopkins University       	 */
+/* Copyright (c) 1998  Tim Edwards, Johns Hopkins University        	 */
 /*-----------------------------------------------------------------------*/
 
 #include <X11/Intrinsic.h>
@@ -31,7 +31,7 @@ extern void UDrawLines();
 extern float UTopScale();
 extern int *appcolors;
 
-#define HALFSPACE 3
+#define HSPACE 3
 
 /*---------------------------------------------------------------------*/
 /* draw a single character at 0, 0 using current transformation matrix */
@@ -75,7 +75,7 @@ short UDrawChar(localdata, code, styles)
       else if (styles & 32)
          alphapts[0].y = alphapts[1].y = TEXTHEIGHT + 4;
       if (styles & 56) {
-         alphapts[0].x = -HALFSPACE; alphapts[1].x = localwidth;
+         alphapts[0].x = -HSPACE; alphapts[1].x = localwidth;
          UDrawSimpleLine(localdata, &alphapts[0], &alphapts[1]);
       }
    }
@@ -201,6 +201,10 @@ void UDrawString(localdata, drawlabel)
 	        backct = ((short)(*(strptr + 1)) != TEXT_ESC || (short)(*(strptr + 2))
 	    	!= BACKSPACE) ? 1 : backct + 1;
              }
+	     else if (control == HALFSPACE || control == QTRSPACE) {
+		short addx = UDrawChar(localdata, 32, (fstyle & 1) | ffont | fline);
+		newpoint.x += addx >> ((control == HALFSPACE) ? 1 : 2);
+	     }
           }
           else
 	     newpoint.x += UDrawChar(localdata, *strptr, (fstyle & 1) | ffont | fline)
@@ -215,6 +219,12 @@ void UDrawString(localdata, drawlabel)
    /* pop the string transformation matrix */
 
    UPopCTM();
+
+#ifdef SCHEMA
+   if (drawlabel->pin) {
+      UDrawX(drawlabel);
+   }
+#endif
 }
 
 short ULength(string, newscale, dostop)
@@ -240,7 +250,13 @@ short ULength(string, newscale, dostop)
 
          if (control == SUPERSCRIPT || control == SUBSCRIPT) strscale *= SUBSCALE;
          else if (control == NORMALSCRIPT) strscale = oldscale;
-         else if (control == BACKSPACE) {
+	 else if (control == HALFSPACE) {
+	    xtotal += (float)((*(somebet + 32))->width) * strscale / 2;
+	 }      
+	 else if (control == QTRSPACE) {
+	    xtotal += (float)((*(somebet + 32))->width) * strscale / 4;
+	 }
+	 else if (control == BACKSPACE) {
 	    if (strptr + 1 - (backct * 3) >= string) {
 	       if (somebet == courierbet)
 	          xtotal -= COURIER_WIDTH * strscale;
@@ -270,7 +286,7 @@ short ULength(string, newscale, dostop)
 	    xtotal += (float)((*(somebet + *strptr))->width) * strscale;
       }
    }
-   return ((short)(xtotal - (HALFSPACE * oldscale)));
+   return ((short)(xtotal - (HSPACE * oldscale)));
 }
 
 /*------------------------------------------------------------------------*/
