@@ -1,6 +1,5 @@
 /*-------------------------------------------------------------------------*/
 /* xcircuit.h 								   */
-/* Copyright (c) 1998  Tim Edwards, Johns Hopkins University       	   */
 /*-------------------------------------------------------------------------*/
 
 /*-------------------------------------------------------------------------*/
@@ -86,9 +85,7 @@
 		   
 /*-------------------------------------------------------------------------*/
 
-#ifndef HAVE_UCHAR
 typedef unsigned char		uchar;
-#endif
 
 typedef struct {
    float x, y;
@@ -128,12 +125,9 @@ typedef struct {
 
 /*-----------------------------------------------------------------------*/
 
-#define INCHSCALE 0.375     /* A normalizing factor for the output scale */
-#define CMSCALE  0.35433071 /* Output scale which makes drawing figures  */
-			    /* defined in inches come out to clean 0.4cm */
-			    /* unit lengths.				 */
-
-#define IN_CM_CONVERT 28.3464567  /* 72 (in) / 2.54 (cm/in) metric conversion */
+#define INCHSCALE 0.375     /* Scale of .25 inches to PostScript units */
+#define CMSCALE 0.35433071  /* Scale of .5 cm to PostScript units */
+#define IN_CM_CONVERT 28.3464567	/* 72 (in) / 2.54 (cm/in) */
 
 /*-------------------------------------------------------------------------*/
 /* Event mode definitions (state of drawing area)			   */
@@ -145,29 +139,30 @@ typedef struct {
 #define DELETE_MODE 	3
 #define COPY_MODE 	4 
 #define COPY2_MODE 	5 
-#define ROTATE_MODE	6
-#define PUSH_MODE 	7 
-#define EDIT_MODE	8 
-#define SELAREA_MODE	9 
+#define ROTATE_MODE     6
+#define PUSH_MODE       7
+#define EDIT_MODE       8
+#define SELAREA_MODE    9
 #define SELAREA2_MODE   10
-#define DESEL_MODE	11
-#define PAN_MODE 	12
-#define CATPAN_MODE 	13
-#define CATALOG_MODE 	14
-#define CATTEXT_MODE	15
-#define TEXT0_MODE 	16
-#define TEXT2_MODE 	17
-#define TEXT3_MODE 	18
-#define WIRE_MODE 	19
-#define BOX0_MODE 	20
-#define BOX_MODE 	21
-#define EBOX_MODE	22
-#define ARC0_MODE	23
-#define ARC_MODE	24
-#define EARC_MODE	25
-#define SPLINE0_MODE	26
-#define SPLINE_MODE	27
-#define ESPLINE_MODE	28
+#define DESEL_MODE      11
+#define PAN_MODE        12
+#define CATPAN_MODE     13
+#define CATALOG_MODE    14
+#define CATTEXT_MODE    15
+#define TEXT0_MODE      16
+#define TEXT1_MODE	17
+#define TEXT2_MODE      18
+#define TEXT3_MODE      19
+#define WIRE_MODE       20 
+#define BOX0_MODE       21
+#define BOX_MODE        22
+#define EBOX_MODE       23
+#define ARC0_MODE       24
+#define ARC_MODE        25
+#define EARC_MODE       26
+#define SPLINE0_MODE    27
+#define SPLINE_MODE     28
+#define ESPLINE_MODE    29
 
 /*-------------------------------------------------------------------------*/
 /* Text justification styles						   */
@@ -263,6 +258,21 @@ typedef struct {
 #define ERASE 1
 #define DRAW  1
 
+/*-------------------------------------------------------------------------*/
+/* Schematic object types and pin label types	   			   */
+/*-------------------------------------------------------------------------*/
+
+#ifdef SCHEMA
+
+#define SCHEMATIC 0
+#define FUNDAMENTAL 1
+#define SYMBOL 2
+
+#define LOCAL 1
+#define GLOBAL 2
+
+#endif
+
 /*-------------------------------------------------------------------------*/
 /* Color scheme styles (other than NORMAL)				   */
 /*-------------------------------------------------------------------------*/
@@ -279,7 +289,7 @@ typedef struct {
 #define CROSS 		appcursors[1]
 #define SCISSORS 	appcursors[2]
 #define COPYCURSOR	appcursors[3]
-#define ROTATECURSOR	appcursors[4]
+#define ROTATECURSOR    appcursors[4]
 #define EDCURSOR	appcursors[5]
 #define TEXTPTR 	appcursors[6]
 #define CIRCLE		appcursors[7]
@@ -344,6 +354,9 @@ typedef struct {
    short	rotation;
    float	scale;
    short	justify;
+#ifdef SCHEMA
+   uchar	pin;
+#endif
    uchar	*string;
 } label, *labelptr;
 
@@ -390,6 +403,43 @@ typedef struct {
    genericptr	*plist;		/* to be retypecast to polygon, arc, or spline */
 } path, *pathptr;
 
+/*--------------------------------------------------------------------------*/
+/* Netlist structures for schematic capture				    */
+/* (courtesy of Chow Seong Hwai, 3/20/98)				    */
+/*--------------------------------------------------------------------------*/
+
+#ifdef SCHEMA
+
+struct Netlist
+{
+   polyptr poly;
+   int netid;
+   struct Netlist *next;
+};
+
+struct Pinlist
+{
+   labelptr pin;
+   char *pinx;
+   int netid;
+   struct Pinlist *next;
+};
+
+struct Calllist
+{
+   objectptr callobj;
+   int callno;
+   int netid;
+   labelptr pin;
+   struct Calllist *next;
+};
+
+#endif
+
+/*--------------------------------------------------------------------------*/
+/* Main object structure						    */
+/*--------------------------------------------------------------------------*/
+
 typedef struct _object {
    char		name[80];
    Boolean	hidden;
@@ -397,6 +447,18 @@ typedef struct _object {
    Dimension	width, height;  /* information  */
    short	parts;
    genericptr	*plist;		/* to be retypecast to label, polygon, etc. */
+#ifdef SCHEMA
+   XPoint	lleft2;		 /* bounding box info excluding pins */
+   Dimension	width2, height2;
+
+   uchar	schemtype;
+   objectptr	symschem;	/* schematic page support */
+
+   struct Netlist   *netlist;
+   struct Pinlist   *pinlist;
+   struct Calllist  *calllist;
+   short traversed;
+#endif
 } object;
 
 /*--------------------------------------------------------------------------*/
@@ -540,11 +602,14 @@ typedef struct {
    Boolean	manhatn;
    Boolean	boxedit;
    Boolean	snapto;
+   short	snapspace;
    Boolean	gridon;
+   short	gridspace;
    Boolean	axeson;
    Boolean	invert;
-   short	snapspace;
-   short	gridspace;
+#ifdef SCHEMA
+   Boolean	schemon;
+#endif
 
    /* buffers, pages, and associated variables */
    XPoint	save, origin;
